"""
All Environment variables
"""
VERSION = "1.1.2"

NOT_VALID_JWT = "Enter a valid JWT!!!"
CHECK_DOCS = "Check Docs!!Your jwt have not a HS alg."
NOT_CRAKED = "JWT not cracked sorry. :'("
CRACKED = "JWT cracked, key is: "
VALID_PAYLOAD = "Enter a Valid payload, Format: username=admin"
VALID_COOKIES = "Enter a Valid cookie, Format: username=admin"
VALID_DATA = "Enter a Valid data, Format: username=admin"
VALID_HEADER = "Enter a Valid header, Format: username=admin"
VALID_PAYLOAD_JSON = "Not a valid format for payload, send a json."
NEW_JWT = "new JWT: "
VALID_SIGNATURE = "Valid Signature!!"
INVALID_SIGNATURE = "Incorrect signature!!"
